import logging

from django.conf import settings

logger = logging.getLogger(__name__)


def clean_setting(
    name: str,
    default_value: object,
    min_value: int = None,
    max_value: int = None,
    required_type: type = None,
):
    """cleans the input for a custom setting

    Will use `default_value` if settings does not exit or has the wrong type
    or is outside define boundaries (for int only)

    Need to define `required_type` if `default_value` is `None`

    Will assume `min_value` of 0 for int (can be overriden)

    Returns cleaned value for setting
    """
    if default_value is None and not required_type:
        raise ValueError("You must specify a required_type for None defaults")

    if not required_type:
        required_type = type(default_value)

    if min_value is None and required_type == int:
        min_value = 0

    if not hasattr(settings, name):
        cleaned_value = default_value
    else:
        if (
            isinstance(getattr(settings, name), required_type)
            and (min_value is None or getattr(settings, name) >= min_value)
            and (max_value is None or getattr(settings, name) <= max_value)
        ):
            cleaned_value = getattr(settings, name)
        else:
            logger.warning(
                "You setting for %s it not valid. Please correct it. "
                "Using default for now: %s",
                name,
                default_value,
            )
            cleaned_value = default_value
    return cleaned_value


AAXENFORO2_ENDPOINT = clean_setting("AAXENFORO2_ENDPOINT", "")
AAXENFORO2_APIKEY = clean_setting("AAXENFORO2_APIKEY", "")
AAXENFORO2_API_USER_ID = clean_setting("AAXENFORO2_API_USER_ID", "1")
AAXENFORO2_DEFAULT_GROUP = clean_setting("AAXENFORO2_DEFAULT_GROUP", "5")
AAXENFORO2_EXTRA_GROUP = clean_setting("AAXENFORO2_EXTRA_GROUP", "2")



from django.conf.urls import include, url

from . import views

app_name = "aaxenforo2"

module_urls = [
    # XenForo2 service control
    url(r"^activate/$", views.activate_xenforo2_forum, name="activate"),
    url(r"^deactivate/$", views.deactivate_xenforo2_forum, name="deactivate"),
    url(r"^reset_password/$", views.reset_xenforo2_password, name="reset_password"),
    url(r"^set_password/$", views.set_xenforo2_password, name="set_password"),
]

urlpatterns = [
    url(r"^aaxenforo2/", include((module_urls, app_name), namespace=app_name)),
]



from django.apps import AppConfig

from aaxenforo2 import __version__


class AAXenforo2ServiceConfig(AppConfig):
    name = "aaxenforo2"
    label = "aaxenforo2"
    verbose_name = f"AA XenForo 2.x Services Integration v{__version__}"



from django.db import models


class AAXenForo2User(models.Model):
    user = models.OneToOneField(
        "auth.User",
        primary_key=True,
        on_delete=models.CASCADE,
        related_name="aaxenforo2",
    )
    remote_user = models.BigIntegerField()
    username = models.CharField(max_length=254)

    def __str__(self):
        return self.username

    class Meta:
        permissions = (("access_xenforo2", "Can access the XenForo2 service"),)



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect, render
from django.utils.translation import gettext_lazy as _

from allianceauth.services.forms import ServicePasswordForm

from .manager import AAXenForo2Manager
from .models import AAXenForo2User
from .tasks import AAXenforo2Tasks

logger = logging.getLogger(__name__)

ACCESS_PERM = "aaxenforo2.access_xenforo2"


@login_required
@permission_required(ACCESS_PERM)
def activate_xenforo2_forum(request):
    logger.debug(f"activate_xenforo2_forum called by user {request.user}")
    character = request.user.profile.main_character
    logger.debug(
        f"Adding XenForo2 user for user {request.user} with main character {character}"
    )
    result = AAXenForo2Manager.add_user(request)
    # Based on XenAPI's response codes
    if result["response"]["status_code"] == 200:
        AAXenForo2User.objects.update_or_create(
            user=request.user,
            defaults={"username": result["username"], "remote_user": result["user_id"]},
        )

        logger.info(
            f"Updated user {request.user} with XenForo2 credentials. Updating groups."
        )
        messages.success(request, _("Activated XenForo2 account."))
        credentials = {
            "username": result["username"],
            "password": result["password"],
        }
        return render(
            request,
            "services/service_credentials.html",
            context={"credentials": credentials, "service": "XenForo2"},
        )

    else:
        logger.error(
            f"Unsuccessful attempt to activate xenforo2 for user {request.user}"
        )
        messages.error(
            request, _("An error occurred while processing your XenForo2 account.")
        )
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_xenforo2_forum(request):
    logger.debug(f"deactivate_xenforo2_forum called by user {request.user}")
    if AAXenforo2Tasks.delete_user(request.user):
        logger.info(f"Successfully deactivated XenForo2 for user {request.user}")
        messages.success(request, _("Deactivated XenForo2 account."))
    else:
        messages.error(
            request, _("An error occurred while processing your XenForo2 account.")
        )
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_xenforo2_password(request):
    logger.debug(f"reset_xenforo2_password called by user {request.user}")
    if AAXenforo2Tasks.has_account(request.user):
        result = AAXenForo2Manager.reset_password(request.user.aaxenforo2)
        # Based on XenAPI's response codes
        if result["response"]["status_code"] == 200:
            logger.info(f"Successfully reset XenForo2 password for user {request.user}")
            messages.success(request, _("Reset XenForo2 account password."))
            credentials = {
                "username": request.user.aaxenforo2.username,
                "password": result["password"],
            }
            return render(
                request,
                "services/service_credentials.html",
                context={"credentials": credentials, "service": "XenForo2"},
            )
    logger.error(
        f"Unsuccessful attempt to reset XenForo2 password for user {request.user}"
    )
    messages.error(
        request, _("An error occurred while processing your XenForo2 account.")
    )
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def set_xenforo2_password(request):
    logger.debug(f"set_xenforo2_password called by user {request.user}")
    if request.method == "POST":
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug(f"Form is valid: {form.is_valid()}")
        if form.is_valid() and AAXenforo2Tasks.has_account(request.user):
            password = form.cleaned_data["password"]
            logger.debug(f"Form contains password of length {len(password)}")
            result = AAXenForo2Manager.update_user_password(
                request.user.aaxenforo2, password
            )

            if result["response"]["status_code"] == 200:
                logger.info(
                    f"Successfully reset XenForo2 password for user {request.user}"
                )
                messages.success(request, _("Changed XenForo2 password."))
            else:
                status_code = result["response"]["status_code"]
                message = result["response"]["message"]
                logger.error(
                    f"Failed to install custom XenForo2 password for user {request.user}: {status_code} {message}"
                )
                messages.error(
                    request,
                    _("An error occurred while processing your XenForo2 account."),
                )
            return redirect("services:services")
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug(f"Rendering form for user {request.user}")
    context = {"form": form, "service": "Forum"}
    return render(request, "services/service_password.html", context=context)



from django.contrib import admin

from allianceauth.services.admin import ServicesUserAdmin

from .models import AAXenForo2User


@admin.register(AAXenForo2User)
class Xenforo2UserAdmin(ServicesUserAdmin):
    list_display = ServicesUserAdmin.list_display + ("username",)
    search_fields = ServicesUserAdmin.search_fields + ("username",)



import logging

from django.core.exceptions import ObjectDoesNotExist

from allianceauth.notifications import notify
from allianceauth.services.hooks import NameFormatter

from .manager import AAXenForo2Manager
from .models import AAXenForo2User

logger = logging.getLogger(__name__)


class AAXenforo2Tasks:
    def __init__(self):
        pass

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user):
            logger.debug(
                f"User {user} has a XenForo2 account {user.aaxenforo2.username}. Deleting."
            )
            if AAXenForo2Manager.disable_user(user.aaxenforo2) == 200:
                user.aaxenforo2.delete()
                if notify_user:
                    notify(user, "XenForo2 Account Disabled", level="danger")
                return True
        return False

    @staticmethod
    def has_account(user):
        try:
            return user.aaxenforo2.username != ""
        except ObjectDoesNotExist:
            return False

    @classmethod
    def disable(cls):
        logger.debug("Deleting ALL XenForo2 users")
        AAXenForo2User.objects.all().delete()

    @staticmethod
    def get_username(user):
        from .auth_hooks import AAXenforo2Service

        return NameFormatter(AAXenforo2Service(), user).format_name()



import logging

from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook

from .tasks import AAXenforo2Tasks
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class AAXenforo2Service(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = "aaxenforo2"
        self.urlpatterns = urlpatterns
        self.access_perm = "aaxenforo2.access_xenforo2"
        self.name_format = "[{corp_ticker}] {character_name}"

    @property
    def title(self):
        return "XenForo 2 Forums"

    def delete_user(self, user, notify_user=False):
        logger.debug(f"Deleting user {user} {self.name} account")
        return AAXenforo2Tasks.delete_user(user, notify_user=notify_user)

    def validate_user(self, user):
        logger.debug(f"Validating user {user} {self.name} account")
        if AAXenforo2Tasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        urls = self.Urls()
        urls.auth_activate = "aaxenforo2:activate"
        urls.auth_deactivate = "aaxenforo2:deactivate"
        urls.auth_reset_password = "aaxenforo2:reset_password"
        urls.auth_set_password = "aaxenforo2:set_password"
        return render_to_string(
            self.service_ctrl_template,
            {
                "service_name": self.title,
                "urls": urls,
                "service_url": "",
                "username": request.user.aaxenforo2.username
                if AAXenforo2Tasks.has_account(request.user)
                else "",
            },
            request=request,
        )


@hooks.register("services_hook")
def register_service():
    return AAXenforo2Service()



import json
import logging
import secrets
import string

import requests

from django.contrib.auth.models import User

from allianceauth.services.hooks import NameFormatter

from .app_settings import (
    AAXENFORO2_API_USER_ID,
    AAXENFORO2_APIKEY,
    AAXENFORO2_DEFAULT_GROUP,
    AAXENFORO2_ENDPOINT,
    AAXENFORO2_EXTRA_GROUP,
)
from .models import AAXenForo2User

logger = logging.getLogger(__name__)


def get_username(user):
    from .auth_hooks import AAXenforo2Service

    return NameFormatter(AAXenforo2Service(), user).format_name()


class AAXenForo2Manager:
    def __init__(self):
        if not AAXENFORO2_ENDPOINT:
            logger.debug("Could not find AAXenForo2 endpoint")
        if not AAXENFORO2_APIKEY:
            logger.debug("AAXenForo2 API Key not found")
        pass

    @staticmethod
    def __sanitize_username(username):
        sanitized = username.replace(" ", "_")
        return sanitized

    @staticmethod
    def __generate_password():
        return "".join(
            [secrets.choice(string.ascii_letters + string.digits) for i in range(12)]
        )

    @staticmethod
    def exec_http_request(route: str, http_params: dict):
        return requests.post(
            f"{AAXENFORO2_ENDPOINT}{route}",
            params=http_params,
            headers={
                "XF-Api-Key": AAXENFORO2_APIKEY,
                "XF-Api-User": AAXENFORO2_API_USER_ID,
            },
        )

    @staticmethod
    def add_user(request):

        user: User = request.user
        username: str = get_username(user)

        data = {
            "username": AAXenForo2Manager.__sanitize_username(username),
            "password": AAXenForo2Manager.__generate_password(),
            "email": user.email,
            "user_group_id": AAXENFORO2_DEFAULT_GROUP,
            "secondary_group_ids[]": AAXENFORO2_EXTRA_GROUP,
            "user_state": "valid",
            "is_staff": "0",
            "visible": "1",
        }

        r = AAXenForo2Manager.exec_http_request("api/users/", data)
        rdata: dict = json.loads(r.text)

        # check if the user already exist but was disabled
        if r.status_code != 200 or not rdata["success"]:
            if any(
                obj["code"] in ["email_addresses_must_be_unique"]
                for obj in rdata["errors"]
            ) and not any(
                obj["code"] in ["usernames_must_be_unique"] for obj in rdata["errors"]
            ):
                # log suspicious behaviour
                msg = f"Duplicate email address used for forum registration by user: [{user.id}] {user.email} ({username})"
                logger.error(msg)

                from django.contrib import messages

                messages.error(
                    request,
                    "Duplicate email address used for forum registration. Please try a different email address. "
                    "(Forums Service Error)",
                )

                raise Exception(msg)
            elif any(
                obj["code"] in ["usernames_must_be_unique"] for obj in rdata["errors"]
            ) and any(
                obj["code"] in ["email_addresses_must_be_unique"]
                for obj in rdata["errors"]
            ):
                # reactivate old account
                data = AAXenForo2Manager.reactivate_user(user.aaxenforo2)
                return data
            elif any(
                obj["code"] in ["usernames_must_be_unique"] for obj in rdata["errors"]
            ):
                # reactivate old account
                data = AAXenForo2Manager.reactivate_user(user.aaxenforo2)
                return data

        response = {
            "user_id": json.loads(r.text)["user"]["user_id"],
            "response": {"message": r.text, "status_code": r.status_code},
        }

        data.update(response)
        return data

    @staticmethod
    def reset_password(user: AAXenForo2User):

        data = {
            "password": AAXenForo2Manager.__generate_password(),
        }

        r = AAXenForo2Manager.exec_http_request(f"api/users/{user.remote_user}/", data)
        rdata = json.loads(r.text)

        # check if the request failed
        if r.status_code != 200 or not rdata["success"]:
            msg = "AAXenForo2 user account password reset failed for user: [{}] {} ({})".format(
                user.user.id, user.user.email, user.username
            )
            logger.error(msg)
            raise Exception(msg)

        response = {"response": {"message": r.text, "status_code": r.status_code}}
        data.update(response)
        return data

    @staticmethod
    def disable_user(user: AAXenForo2User):

        data = {
            "user_group_id": "1",
            "secondary_group_ids": "",
            "user_state": "disabled",
        }

        r = AAXenForo2Manager.exec_http_request(f"api/users/{user.remote_user}/", data)
        rdata = json.loads(r.text)

        # check if the request failed
        if r.status_code != 200 or not rdata["success"]:
            msg = "AAXenForo2 user deactivated for user: [{}] {} ({})".format(
                user.user.id, user.user.email, user.username
            )
            logger.error(msg)
            raise Exception(msg)

        return r

    @staticmethod
    def reactivate_user(user: AAXenForo2User):
        data = {
            "user_group_id": AAXENFORO2_DEFAULT_GROUP,
            "secondary_group_ids[]": AAXENFORO2_EXTRA_GROUP,
            "user_state": "valid",
        }

        r = AAXenForo2Manager.exec_http_request(f"api/users/{user.remote_user}/", data)
        rdata = json.loads(r.text)

        # check if the request failed
        if r.status_code != 200 or not rdata["success"]:
            msg = "AAXenForo2 user reactivated for user: [{}] {} ({})".format(
                user.user.id, user.user.email, user.username
            )
            logger.error(msg)
            raise Exception(msg)

        response = {
            "response": {"message": r.text, "status_code": r.status_code},
            "username": user.username,
        }
        data.update(response)
        return data

    @staticmethod
    def update_user_password(user: AAXenForo2User, raw_password):

        data = {
            "password": raw_password,
        }

        r = AAXenForo2Manager.exec_http_request(f"api/users/{user.remote_user}/", data)
        rdata = json.loads(r.text)

        # check if the request failed
        if r.status_code != 200 or not rdata["success"]:
            msg = "AAXenForo2 user account password update failed for user: [{}] {} ({})".format(
                user.user.id, user.user.email, user.username
            )
            logger.error(msg)
            raise Exception(msg)

        response = {
            "response": {"message": r.text, "status_code": r.status_code},
            "username": user.username,
        }
        data.update(response)
        return data



# Generated by Django 1.10.5 on 2017-02-02 05:59

import logging

from django.conf import settings
from django.contrib.auth.management import create_permissions
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations

logger = logging.getLogger(__name__)


def migrate_service_enabled(apps, schema_editor):
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=0)
        app_config.models_module = None

    Group = apps.get_model("auth", "Group")
    Permission = apps.get_model("auth", "Permission")
    AAXenforo2User = apps.get_model("aaxenforo2", "AAXenforo2User")

    perm = Permission.objects.get(codename="access_xenforo2")

    member_group_name = getattr(settings, "DEFAULT_AUTH_GROUP", "Member")
    blue_group_name = getattr(settings, "DEFAULT_BLUE_GROUP", "Blue")

    # Migrate members
    if AAXenforo2User.objects.filter(
        user__groups__name=member_group_name
    ).exists() or getattr(settings, "ENABLE_AUTH_XENFORO2", False):
        try:
            group = Group.objects.get(name=member_group_name)
            group.permissions.add(perm)
        except ObjectDoesNotExist:
            logger.warning("Failed to migrate ENABLE_AUTH_XENFORO2 setting")

    # Migrate blues
    if AAXenforo2User.objects.filter(
        user__groups__name=blue_group_name
    ).exists() or getattr(settings, "ENABLE_BLUE_XENFORO2", False):
        try:
            group = Group.objects.get(name=blue_group_name)
            group.permissions.add(perm)
        except ObjectDoesNotExist:
            logger.warning("Failed to migrate ENABLE_BLUE_XENFORO2 setting")


class Migration(migrations.Migration):
    dependencies = [
        ("aaxenforo2", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="aaxenforo2user",
            options={
                "permissions": (("access_xenforo2", "Can access the XenForo2 service"),)
            },
        ),
        migrations.RunPython(migrate_service_enabled, migrations.RunPython.noop),
    ]



from unittest import mock

from django import urls
from django.contrib.auth.models import Group, Permission, User
from django.core.exceptions import ObjectDoesNotExist
from django.test import RequestFactory, TestCase

from allianceauth.tests.auth_utils import AuthUtils

from .auth_hooks import AAXenforo2Service
from .models import AAXenForo2User
from .tasks import AAXenforo2Tasks

MODULE_PATH = "aaxenforo2"
DEFAULT_AUTH_GROUP = "Member"


def add_permissions():
    permission = Permission.objects.get(codename="access_xenforo2")
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])


class AAXenforo2HooksTestCase(TestCase):
    def setUp(self):
        self.member = "member_user"
        member = AuthUtils.create_member(self.member)
        AAXenForo2User.objects.create(user=member, username=self.member, remote_user=2)

        self.none_user = "none_user"
        AuthUtils.create_user(self.none_user)

        self.service = AAXenforo2Service
        add_permissions()

    def test_has_account(self):
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)
        self.assertTrue(AAXenforo2Tasks.has_account(member))
        self.assertFalse(AAXenforo2Tasks.has_account(none_user))

    def test_service_enabled(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)

        self.assertTrue(service.service_active_for_user(member))
        self.assertFalse(service.service_active_for_user(none_user))

    @mock.patch(MODULE_PATH + ".tasks.AAXenForo2Manager")
    def test_validate_user(self, manager):
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        manager.disable_user.return_value = 200

        service.validate_user(member)
        self.assertTrue(member.aaxenforo2)

        # Test none user is deleted
        none_user = User.objects.get(username=self.none_user)
        AAXenForo2User.objects.create(user=none_user, username="abc123", remote_user=2)
        service.validate_user(none_user)
        self.assertTrue(manager.disable_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            User.objects.get(username=self.none_user).aaxenforo2

    @mock.patch(MODULE_PATH + ".tasks.AAXenForo2Manager")
    def test_delete_user(self, manager):
        member = User.objects.get(username=self.member)
        manager.disable_user.return_value = 200
        service = self.service()

        result = service.delete_user(member)

        self.assertTrue(result)
        self.assertTrue(manager.disable_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            User.objects.get(username=self.member).aaxenforo2

    def test_render_services_ctrl(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        request = RequestFactory().get("/en/services/")
        request.user = member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn(urls.reverse("aaxenforo2:deactivate"), response)
        self.assertIn(urls.reverse("aaxenforo2:reset_password"), response)
        self.assertIn(urls.reverse("aaxenforo2:set_password"), response)

        # Test register becomes available
        member.aaxenforo2.delete()
        member = User.objects.get(username=self.member)
        request.user = member
        response = service.render_services_ctrl(request)
        self.assertIn(urls.reverse("aaxenforo2:activate"), response)


class AAXenforo2ViewsTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_member("auth_member")
        self.member.email = "auth_member@example.com"
        self.member.save()
        AuthUtils.add_main_character(
            self.member,
            "auth_member",
            "12345",
            corp_id="111",
            corp_name="Test Corporation",
        )
        add_permissions()

    def login(self):
        self.client.force_login(self.member)

    @mock.patch(MODULE_PATH + ".tasks.AAXenForo2Manager")
    @mock.patch(MODULE_PATH + ".views.AAXenForo2Manager")
    def test_activate(self, manager, tasks_manager):
        self.login()
        expected_username = "auth_member"
        manager.add_user.return_value = {
            "response": {"status_code": 200},
            "password": "hunter2",
            "username": expected_username,
            "user_id": 2,
        }

        response = self.client.get(urls.reverse("aaxenforo2:activate"))

        self.assertTrue(manager.add_user.called)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed("services/service_credentials.html")
        self.assertContains(response, expected_username)
        xenforo2_user2 = AAXenForo2User.objects.get(user=self.member)
        self.assertEqual(xenforo2_user2.username, expected_username)

    @mock.patch(MODULE_PATH + ".tasks.AAXenForo2Manager")
    def test_deactivate(self, manager):
        self.login()
        AAXenForo2User.objects.create(
            user=self.member, username="some member", remote_user=2
        )

        manager.disable_user.return_value = 200

        response = self.client.get(urls.reverse("aaxenforo2:deactivate"))

        self.assertTrue(manager.disable_user.called)
        self.assertRedirects(
            response,
            expected_url=urls.reverse("services:services"),
            target_status_code=200,
        )
        with self.assertRaises(ObjectDoesNotExist):
            User.objects.get(pk=self.member.pk).aaxenforo2

    @mock.patch(MODULE_PATH + ".views.AAXenForo2Manager")
    def test_set_password(self, manager):
        self.login()
        AAXenForo2User.objects.create(
            user=self.member, username="some member", remote_user=2
        )

        manager.update_user_password.return_value = {
            "response": {"status_code": 200, "message": "ok"},
            "username": "some member",
        }

        response = self.client.post(
            urls.reverse("aaxenforo2:set_password"), data={"password": "1234asdf"}
        )

        self.assertTrue(manager.update_user_password.called)
        args, _ = manager.update_user_password.call_args
        self.assertEqual(args[1], "1234asdf")
        self.assertRedirects(
            response,
            expected_url=urls.reverse("services:services"),
            target_status_code=200,
        )

    @mock.patch(MODULE_PATH + ".views.AAXenForo2Manager")
    def test_reset_password(self, manager):
        self.login()
        AAXenForo2User.objects.create(
            user=self.member, username="some member", remote_user=2
        )

        manager.reset_password.return_value = {
            "response": {"status_code": 200, "message": "ok"},
            "password": "hunter2",
        }

        response = self.client.get(urls.reverse("aaxenforo2:reset_password"))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "services/service_credentials.html")
        self.assertContains(response, "some member")
        self.assertContains(response, "hunter2")


class AAXenforo2ManagerTestCase(TestCase):
    def setUp(self):
        from .manager import AAXenForo2Manager

        self.manager = AAXenForo2Manager

    def test_generate_password(self):
        password = self.manager._AAXenForo2Manager__generate_password()

        self.assertEqual(len(password), 12)
        self.assertIsInstance(password, str)



# Generated by Django 1.10.2 on 2016-12-12 03:14

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0008_alter_user_username_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="AAXenforo2User",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        related_name="aaxenforo2",
                        serialize=False,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("username", models.CharField(max_length=254)),
                ("remote_user", models.BigIntegerField(default=0)),
            ],
        ),
    ]
